@@ -1,26 +1,27 @@
-function [fval,llik,cost_flag,ys,trend_coeff,info] = DsgeLikelihood_hh(xparam1,gend,data)
-
-% function [fval,llik,cost_flag,ys,trend_coeff,info] = DsgeLikelihood_hh(xparam1,gend,data)
-% Evaluates the likelihood at each observation and the marginal density of a dsge model  
-% used in the optimization algorithm number 5
+function [fval,llik,cost_flag,ys,trend_coeff,info] = DsgeLikelihood(xparam1,gend,data,data_index,number_of_observations,no_more_missing_observations)
+% function [fval,cost_flag,ys,trend_coeff,info] = DsgeLikelihood(xparam1,gend,data,data_index,number_of_observations,no_more_missing_observations)
+% Evaluates the posterior kernel of a dsge model. 
 % 
 % INPUTS 
-%   xparam1:          vector of model parameters
-%   gend   :          scalar specifying the number of observations
-%   data   :          matrix of data
-%  
+%   xparam1                        [double]   vector of model parameters.
+%   gend                           [integer]  scalar specifying the number of observations.
+%   data                           [double]   matrix of data
+%   data_index                     [cell]     cell of column vectors
+%   number_of_observations         [integer]
+%   no_more_missing_observations   [integer] 
 % OUTPUTS 
-%   fval        :     value of the posterior kernel at xparam1
-%   llik        :     gives the density at each observation
-%   cost_flag   :     zero if the function returns a penalty, one otherwise
+%   fval        :     value of the posterior kernel at xparam1.
+%   cost_flag   :     zero if the function returns a penalty, one otherwise.
 %   ys          :     steady state of original endogenous variables
 %   trend_coeff :
-%   info        :     vector of informations about the penalty                
+%   info        :     vector of informations about the penalty:
+%                     41: one (many) parameter(s) do(es) not satisfied the lower bound
+%                     42: one (many) parameter(s) do(es) not satisfied the upper bound
 %               
 % SPECIAL REQUIREMENTS
-%   Adapted from dsgelikelihood.m
+%
 
-% Copyright (C) 2006-2009 Dynare Team
+% Copyright (C) 2004-2009 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -37,32 +38,29 @@ function [fval,llik,cost_flag,ys,trend_coeff,info] = DsgeLikelihood_hh(xparam1,g
 % You should have received a copy of the GNU General Public License
 % along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
 
-  global bayestopt_ estim_params_ options_ trend_coeff_ M_ oo_ xparam1_test
-
+  global bayestopt_ estim_params_ options_ trend_coeff_ M_ oo_
   fval		= [];
   ys		= [];
   trend_coeff	= [];
-  xparam1_test  = xparam1;
   cost_flag  	= 1;
   nobs 		= size(options_.varobs,1);
+  llik=NaN;
   %------------------------------------------------------------------------------
   % 1. Get the structural parameters & define penalties
   %------------------------------------------------------------------------------
   if options_.mode_compute ~= 1 & any(xparam1 < bayestopt_.lb)
-    k = find(xparam1 < bayestopt_.lb);
-    fval = bayestopt_.penalty+sum((bayestopt_.lb(k)-xparam1(k)).^2);
-    llik=fval;
-    cost_flag = 0;
-    info = 41;
-    return;
+      k = find(xparam1 < bayestopt_.lb);
+      fval = bayestopt_.penalty+sum((bayestopt_.lb(k)-xparam1(k)).^2);
+      cost_flag = 0;
+      info = 41;
+      return;
   end
   if options_.mode_compute ~= 1 & any(xparam1 > bayestopt_.ub)
-    k = find(xparam1 > bayestopt_.ub);
-    fval = bayestopt_.penalty+sum((xparam1(k)-bayestopt_.ub(k)).^2);
-    llik=fval;
-    cost_flag = 0;
-    info = 42;
-    return;
+      k = find(xparam1 > bayestopt_.ub);
+      fval = bayestopt_.penalty+sum((xparam1(k)-bayestopt_.ub(k)).^2);
+      cost_flag = 0;
+      info = 42;
+      return;
   end
   Q = M_.Sigma_e;
   H = M_.H;
@@ -77,7 +75,7 @@ function [fval,llik,cost_flag,ys,trend_coeff,info] = DsgeLikelihood_hh(xparam1,g
       H(k,k) = xparam1(i+offset)*xparam1(i+offset);
     end
     offset = offset+estim_params_.nvn;
-  end	
+  end
   if estim_params_.ncx
     for i=1:estim_params_.ncx
       k1 =estim_params_.corrx(i,1);
@@ -92,7 +90,6 @@ function [fval,llik,cost_flag,ys,trend_coeff,info] = DsgeLikelihood_hh(xparam1,g
 		k = find(a < 0);
 		if k > 0
 		  fval = bayestopt_.penalty+sum(-a(k));
-      llik=fval;
 		  cost_flag = 0;
 		  info = 43;
 		  return
@@ -113,7 +110,6 @@ function [fval,llik,cost_flag,ys,trend_coeff,info] = DsgeLikelihood_hh(xparam1,g
       k = find(a < 0);
       if k > 0
 	fval = bayestopt_.penalty+sum(-a(k));
-  llik=fval;
 	cost_flag = 0;
 	info = 44;
 	return
@@ -121,10 +117,9 @@ function [fval,llik,cost_flag,ys,trend_coeff,info] = DsgeLikelihood_hh(xparam1,g
     end
     offset = offset+estim_params_.ncn;
   end
-  M_.params(estim_params_.param_vals(:,1)) = xparam1(offset+1:end);  
-  % for i=1:estim_params_.np
-  %  M_.params(estim_params_.param_vals(i,1)) = xparam1(i+offset);
-  %end
+  if estim_params_.np > 0
+      M_.params(estim_params_.param_vals(:,1)) = xparam1(offset+1:end);
+  end
   M_.Sigma_e = Q;
   M_.H = H;
   %------------------------------------------------------------------------------
@@ -133,28 +128,26 @@ function [fval,llik,cost_flag,ys,trend_coeff,info] = DsgeLikelihood_hh(xparam1,g
   [T,R,SteadyState,info] = dynare_resolve(bayestopt_.restrict_var_list,...
 					  bayestopt_.restrict_columns,...
 					  bayestopt_.restrict_aux);
-  if info(1) == 1 | info(1) == 2 | info(1) == 5
-    fval = bayestopt_.penalty+1;
-    llik=fval;
-    cost_flag = 0;
-    return
-  elseif info(1) == 3 | info(1) == 4 | info(1) == 20
-    fval = bayestopt_.penalty+info(2)^2;
-    llik=fval;
-    cost_flag = 0;
-    return
+  if info(1) == 1 || info(1) == 2 || info(1) == 5
+      fval = bayestopt_.penalty+1;
+      cost_flag = 0;
+      return
+  elseif info(1) == 3 || info(1) == 4 || info(1)==6 ||info(1) == 19 || info(1) == 20 || info(1) == 21
+      fval = bayestopt_.penalty+info(2);
+      cost_flag = 0;
+      return
   end
   bayestopt_.mf = bayestopt_.mf1;
-  if ~options_.noconstant
-    if options_.loglinear == 1
-      constant = log(SteadyState(bayestopt_.mfys));
-    else
-      constant = SteadyState(bayestopt_.mfys);
-    end
-  else
-    constant = zeros(nobs,1);
+  if options_.noconstant
+      constant = zeros(nobs,1);  
+  else    
+      if options_.loglinear
+          constant = log(SteadyState(bayestopt_.mfys));
+      else
+          constant = SteadyState(bayestopt_.mfys);
+      end
   end
-  if bayestopt_.with_trend == 1
+  if bayestopt_.with_trend
     trend_coeff = zeros(nobs,1);
     t = options_.trend_coeffs;
     for i=1:length(t)
@@ -169,31 +162,31 @@ function [fval,llik,cost_flag,ys,trend_coeff,info] = DsgeLikelihood_hh(xparam1,g
   start = options_.presample+1;
   np    = size(T,1);
   mf    = bayestopt_.mf;
+  no_missing_data_flag = (number_of_observations==gend*nobs);
   %------------------------------------------------------------------------------
   % 3. Initial condition of the Kalman filter
   %------------------------------------------------------------------------------
+  kalman_algo = options_.kalman_algo;
   if options_.lik_init == 1		% Kalman filter
-    Pstar = lyapunov_symm(T,R*Q*R',options_.qz_criterium);
-    Pinf	= [];
+      if kalman_algo ~= 2
+          kalman_algo = 1;
+      end
+      Pstar = lyapunov_symm(T,R*Q*R',options_.qz_criterium,options_.lyapunov_complex_threshold);
+      Pinf	= [];
   elseif options_.lik_init == 2	% Old Diffuse Kalman filter
-    Pstar = 10*eye(np);
-    Pinf	= [];
+      if kalman_algo ~= 2
+          kalman_algo = 1;
+      end
+      Pstar = options_.Harvey_scale_factor*eye(np);
+      Pinf = [];
   elseif options_.lik_init == 3	% Diffuse Kalman filter
       if kalman_algo ~= 4
           kalman_algo = 3;
       end
       [QT,ST] = schur(T);
-      if exist('OCTAVE_VERSION') || matlab_ver_less_than('7.0.1')
-          e1 = abs(my_ordeig(ST)) > 2-options_.qz_criterium;
-      else
-          e1 = abs(ordeig(ST)) > 2-options_.qz_criterium;
-      end
+      e1 = abs(ordeig(ST)) > 2-options_.qz_criterium;
       [QT,ST] = ordschur(QT,ST,e1);
-      if exist('OCTAVE_VERSION') || matlab_ver_less_than('7.0.1')
-          k = find(abs(my_ordeig(ST)) > 2-options_.qz_criterium);
-      else
-          k = find(abs(ordeig(ST)) > 2-options_.qz_criterium);
-      end
+      k = find(abs(ordeig(ST)) > 2-options_.qz_criterium);
       nk = length(k);
       nk1 = nk+1;
       Pinf = zeros(np,np);
@@ -248,52 +241,89 @@ function [fval,llik,cost_flag,ys,trend_coeff,info] = DsgeLikelihood_hh(xparam1,g
 	  Pinf(1:nk,1:nk) = diag(dd);
       end
   end
+  if kalman_algo == 2
+  end
+  kalman_tol = options_.kalman_tol;
+  riccati_tol = options_.riccati_tol;
+  mf = bayestopt_.mf1;
+  Y   = data-trend;
   %------------------------------------------------------------------------------
   % 4. Likelihood evaluation
   %------------------------------------------------------------------------------
-  if any(any(H ~= 0)) % should be replaced by a flag
-    if options_.kalman_algo == 1
-      [LIK, lik] =DiffuseLikelihoodH1(T,R,Q,H,Pinf,Pstar,data,trend,start);
-      if isinf(LIK) & ~estim_params_.ncn %% The univariate approach considered here doesn't 
-					 %%	apply when H has some off-diagonal elements.
-        [LIK, lik] =DiffuseLikelihoodH3(T,R,Q,H,Pinf,Pstar,data,trend,start);
-      elseif isinf(LIK) & estim_params_.ncn
-	[LIK, lik] =DiffuseLikelihoodH3corr(T,R,Q,H,Pinf,Pstar,data,trend,start);
-      end
-    elseif options_.kalman_algo == 3
-      if ~estim_params_.ncn %% The univariate approach considered here doesn't 
-			    %%	apply when H has some off-diagonal elements.
-        [LIK, lik] =DiffuseLikelihoodH3(T,R,Q,H,Pinf,Pstar,data,trend,start);
+  if (kalman_algo==1)% Multivariate Kalman Filter
+      if no_missing_data_flag
+          [LIK, lik] = kalman_filter(T,R,Q,H,Pstar,Y,start,mf,kalman_tol,riccati_tol); 
       else
-	[LIK, lik] =DiffuseLikelihoodH3corr(T,R,Q,H,Pinf,Pstar,data,trend,start);
-      end	
-    end	  
-  else
-    if options_.kalman_algo == 1
-       %nv = size(bayestopt_.Z,1);
-       %LIK = kalman_filter(bayestopt_.Z,zeros(nv,nv),T,R,Q,data,zeros(size(T,1),1),Pstar,'u');
-      [LIK, lik] =DiffuseLikelihood1(T,R,Q,Pinf,Pstar,data,trend,start);
-      % LIK = diffuse_likelihood1(T,R,Q,Pinf,Pstar,data-trend,start);
-      %if abs(LIK1-LIK)>0.0000000001
-      %  disp(['LIK1 and LIK are not equal! ' num2str(abs(LIK1-LIK))])
-      %end
+          [LIK, lik] = ...
+              missing_observations_kalman_filter(T,R,Q,H,Pstar,Y,start,mf,kalman_tol,riccati_tol, ...
+                                                 data_index,number_of_observations,no_more_missing_observations);
+      end
       if isinf(LIK)
-	[LIK, lik] =DiffuseLikelihood3(T,R,Q,Pinf,Pstar,data,trend,start);
+          kalman_algo = 2;
+      end
+  end
+  if (kalman_algo==2)% Univariate Kalman Filter
+      no_correlation_flag = 1;
+      if length(H)==1 & H == 0
+          H = zeros(nobs,1);
+      else
+          if all(all(abs(H-diag(diag(H)))<1e-14))% ie, the covariance matrix is diagonal...
+              H = diag(H);
+          else
+              no_correlation_flag = 0;
+          end
+      end
+      if no_correlation_flag
+          [LIK, lik] = univariate_kalman_filter(T,R,Q,H,Pstar,Y,start,mf,kalman_tol,riccati_tol,data_index,number_of_observations,no_more_missing_observations);
+      else
+          [LIK, lik] = univariate_kalman_filter_corr(T,R,Q,H,Pstar,Y,start,mf,kalman_tol,riccati_tol,data_index,number_of_observations,no_more_missing_observations);
+      end
+  end
+  if (kalman_algo==3)% Multivariate Diffuse Kalman Filter
+      if no_missing_data_flag
+          [LIK, lik] = diffuse_kalman_filter(ST,R1,Q,H,Pinf,Pstar,Y,start,Z,kalman_tol, ...
+                                      riccati_tol);
+      else
+          [LIK, lik] = missing_observations_diffuse_kalman_filter(ST,R1,Q,H,Pinf, ...
+                                                           Pstar,Y,start,Z,kalman_tol,riccati_tol,...
+                                                           data_index,number_of_observations,...
+                                                           no_more_missing_observations);
+      end
+      if isinf(LIK)
+          kalman_algo = 4;
+      end
+  end
+  if (kalman_algo==4)% Univariate Diffuse Kalman Filter
+      no_correlation_flag = 1;
+      if length(H)==1 & H == 0
+          H = zeros(nobs,1);
+      else
+          if all(all(abs(H-diag(diag(H)))<1e-14))% ie, the covariance matrix is diagonal...
+              H = diag(H);
+          else
+              no_correlation_flag = 0;
+          end
+      end
+      if no_correlation_flag
+          [LIK, lik] = univariate_diffuse_kalman_filter(ST,R1,Q,H,Pinf,Pstar,Y, ...
+                                                 start,Z,kalman_tol,riccati_tol,data_index,...
+                                                 number_of_observations,no_more_missing_observations);
+      else
+          [LIK, lik] = univariate_diffuse_kalman_filter_corr(ST,R1,Q,H,Pinf,Pstar, ...
+                                                      Y,start,Z,kalman_tol,riccati_tol,...
+                                                      data_index,number_of_observations,...
+                                                      no_more_missing_observations);
       end
-    elseif options_.kalman_algo == 3
-      [LIK, lik] =DiffuseLikelihood3(T,R,Q,Pinf,Pstar,data,trend,start);
-    end 	
   end
   if imag(LIK) ~= 0
-    likelihood = bayestopt_.penalty;
-    lik=ones(size(lik)).*bayestopt_.penalty;
+      likelihood = bayestopt_.penalty;
   else
-    likelihood = LIK;
+      likelihood = LIK;
   end
   % ------------------------------------------------------------------------------
   % Adds prior if necessary
   % ------------------------------------------------------------------------------
-  lnprior = priordens(xparam1,bayestopt_.pshape,bayestopt_.p1,bayestopt_.p2,bayestopt_.p3,bayestopt_.p4);
+  lnprior = priordens(xparam1,bayestopt_.pshape,bayestopt_.p6,bayestopt_.p7,bayestopt_.p3,bayestopt_.p4);
   fval    = (likelihood-lnprior);
-  llik=[-lnprior; .5*lik(start:end)];
-  
+  options_.kalman_algo = kalman_algo;
+  llik=[-lnprior; lik(start:end)];
