@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2008 Dynare Team
+ * Copyright (C) 2003-2009 Dynare Team
  *
  * This file is part of Dynare.
  *
@@ -24,23 +24,15 @@
 #include "ParsingDriver.hh"
 #include "Statement.hh"
 
-ParsingDriver::ParsingDriver()
-{
-}
-
-ParsingDriver::~ParsingDriver()
-{
-}
-
 bool
 ParsingDriver::symbol_exists_and_is_not_modfile_local_or_unknown_function(const char *s)
 {
   if (!mod_file->symbol_table.exists(s))
     return false;
 
-  Type type = mod_file->symbol_table.getType(s);
+  SymbolType type = mod_file->symbol_table.getType(s);
 
-  return(type != eModFileLocalVariable && type != eUnknownFunction);
+  return (type != eModFileLocalVariable && type != eUnknownFunction);
 }
 
 void
@@ -55,6 +47,7 @@ ParsingDriver::set_current_data_tree(DataTree *data_tree_arg)
 {
   data_tree = data_tree_arg;
   model_tree = dynamic_cast<ModelTree *>(data_tree_arg);
+  dynamic_model = dynamic_cast<DynamicModel *>(data_tree_arg);
 }
 
 void
@@ -71,6 +64,7 @@ ParsingDriver::parse(istream &in, bool debug)
   symbol_list.clear();
 
   reset_data_tree();
+  estim_params.init(*data_tree);
 
   lexer = new DynareFlex(&in);
   lexer->set_debug(debug);
@@ -88,7 +82,7 @@ void
 ParsingDriver::error(const Dynare::parser::location_type &l, const string &m)
 {
   cerr << "ERROR: " << l << ": " << m << endl;
-  exit(-1);
+  exit(EXIT_FAILURE);
 }
 
 void
@@ -104,13 +98,16 @@ ParsingDriver::warning(const string &m)
 }
 
 void
-ParsingDriver::declare_symbol(string *name, Type type, string *tex_name)
+ParsingDriver::declare_symbol(string *name, SymbolType type, string *tex_name)
 {
   try
     {
-      mod_file->symbol_table.addSymbol(*name, type, *tex_name);
+      if (tex_name == NULL)
+        mod_file->symbol_table.addSymbol(*name, type);
+      else
+        mod_file->symbol_table.addSymbol(*name, type, *tex_name);
     }
-  catch(SymbolTable::AlreadyDeclaredException &e)
+  catch (SymbolTable::AlreadyDeclaredException &e)
     {
       if (e.same_type)
         warning("Symbol " + *name + " declared twice.");
@@ -119,7 +116,8 @@ ParsingDriver::declare_symbol(string *name, Type type, string *tex_name)
     }
 
   delete name;
-  delete tex_name;
+  if (tex_name != NULL)
+    delete tex_name;
 }
 
 void
@@ -146,6 +144,33 @@ ParsingDriver::declare_parameter(string *name, string *tex_name)
   declare_symbol(name, eParameter, tex_name);
 }
 
+void
+ParsingDriver::add_predetermined_variable(string *name)
+{
+  try
+    {
+      int symb_id = mod_file->symbol_table.getID(*name);
+      if (mod_file->symbol_table.getType(symb_id) != eEndogenous)
+        error("Predetermined variables must be endogenous variables");
+
+      mod_file->symbol_table.markPredetermined(symb_id);
+    }
+  catch (SymbolTable::UnknownSymbolNameException &e)
+    {
+      error("Undeclared symbol name: " + *name);
+    }
+  delete name;
+}
+
+void
+ParsingDriver::add_equation_tags(string *key, string *value)
+{
+  int n = model_tree->equation_number();
+  model_tree->addEquationTags(n, *key, *value);
+  delete key;
+  delete value;
+}
+
 NodeID
 ParsingDriver::add_constant(string *constant)
 {
@@ -154,49 +179,44 @@ ParsingDriver::add_constant(string *constant)
   return id;
 }
 
+NodeID
+ParsingDriver::add_nan_constant()
+{
+  return data_tree->NaN;
+}
+
+NodeID
+ParsingDriver::add_inf_constant()
+{
+  return data_tree->Infinity;
+}
+
 NodeID
 ParsingDriver::add_model_variable(string *name)
 {
-  check_symbol_existence(*name);
-  NodeID id = model_tree->AddVariable(*name);
-
-  Type type = mod_file->symbol_table.getType(*name);
-
-  if (type == eModFileLocalVariable)
-    error("Variable " + *name + " not allowed inside model declaration. Its scope is only outside model.");
-
-  if ((type == eEndogenous) && (model_tree->mode == eSparseDLLMode || model_tree->mode == eSparseMode))
-    {
-      int ID = mod_file->symbol_table.getID(*name);
-      model_tree->block_triangular.fill_IM(model_tree->equation_number(), ID, 0);
-    }
-
-  delete name;
-  return id;
+  return add_model_variable(name, new string("0"));
 }
 
 NodeID
 ParsingDriver::add_model_variable(string *name, string *olag)
 {
   check_symbol_existence(*name);
-  Type type = mod_file->symbol_table.getType(*name);
+  SymbolType type = mod_file->symbol_table.getType(*name);
   int lag = atoi(olag->c_str());
 
   if (type == eModFileLocalVariable)
     error("Variable " + *name + " not allowed inside model declaration. Its scope is only outside model.");
 
-  if ((type == eExogenous) && lag != 0)
-    {
-      ostringstream ost;
-      ost << "Exogenous variable " << *name << " has lag " << lag;
-      warning(ost.str());
-    }
+  if (type == eUnknownFunction)
+    error("Symbol " + *name + " is a function name unknown to Dynare. It cannot be used inside model.");
+
+  if (type == eModelLocalVariable && lag != 0)
+    error("Model local variable " + *name + " cannot be given a lead or a lag.");
+
+  // It makes sense to allow a lead/lag on parameters: during steady state calibration, endogenous and parameters can be swapped
 
   NodeID id = model_tree->AddVariable(*name, lag);
 
-  if ((type == eEndogenous) && (model_tree->mode == eSparseDLLMode || model_tree->mode == eSparseMode))
-    model_tree->block_triangular.fill_IM(model_tree->equation_number(), mod_file->symbol_table.getID(*name), lag);
-
   delete name;
   delete olag;
   return id;
@@ -222,27 +242,13 @@ ParsingDriver::add_expression_variable(string *name)
 void
 ParsingDriver::periods(string *periods)
 {
+  warning("periods: this command is now deprecated and may be removed in a future version of Dynare. Please of the \"periods\" option of \"simul\" command instead.");
+
   int periods_val = atoi(periods->c_str());
   mod_file->addStatement(new PeriodsStatement(periods_val));
   delete periods;
 }
 
-void
-ParsingDriver::cutoff(string *cutoff)
-{
-  double cutoff_val = atof(cutoff->c_str());
-  mod_file->addStatement(new CutoffStatement(cutoff_val));
-  delete cutoff;
-}
-
-void
-ParsingDriver::markowitz(string *markowitz)
-{
-  double markowitz_val = atof(markowitz->c_str());
-  mod_file->addStatement(new MarkowitzStatement(markowitz_val));
-  delete markowitz;
-}
-
 void
 ParsingDriver::dsample(string *arg1)
 {
@@ -265,21 +271,11 @@ void
 ParsingDriver::init_param(string *name, NodeID rhs)
 {
   check_symbol_existence(*name);
-  if (mod_file->symbol_table.getType(*name) != eParameter)
+  int symb_id = mod_file->symbol_table.getID(*name);
+  if (mod_file->symbol_table.getType(symb_id) != eParameter)
     error(*name + " is not a parameter");
 
-  mod_file->addStatement(new InitParamStatement(*name, rhs, mod_file->symbol_table));
-
-  // Update global eval context
-  try
-    {
-      double val = rhs->eval(mod_file->global_eval_context);
-      int symb_id = mod_file->symbol_table.getID(*name);
-      mod_file->global_eval_context[make_pair(symb_id, eParameter)] = val;
-     }
-  catch(ExprNode::EvalException &e)
-    {
-    }
+  mod_file->addStatement(new InitParamStatement(symb_id, rhs, mod_file->symbol_table));
 
   delete name;
 }
@@ -288,25 +284,15 @@ void
 ParsingDriver::init_val(string *name, NodeID rhs)
 {
   check_symbol_existence(*name);
-  Type type = mod_file->symbol_table.getType(*name);
+  int symb_id = mod_file->symbol_table.getID(*name);
+  SymbolType type = mod_file->symbol_table.getType(symb_id);
 
   if (type != eEndogenous
       && type != eExogenous
       && type != eExogenousDet)
     error("initval/endval: " + *name + " should be an endogenous or exogenous variable");
 
-  init_values.push_back(make_pair(*name, rhs));
-
-  // Update global evaluation context
-  try
-    {
-      double val = rhs->eval(mod_file->global_eval_context);
-      int symb_id = mod_file->symbol_table.getID(*name);
-      mod_file->global_eval_context[make_pair(symb_id, type)] = val;
-    }
-  catch(ExprNode::EvalException &e)
-    {
-    }
+  init_values.push_back(make_pair(symb_id, rhs));
 
   delete name;
 }
@@ -322,7 +308,8 @@ void
 ParsingDriver::hist_val(string *name, string *lag, NodeID rhs)
 {
   check_symbol_existence(*name);
-  Type type = mod_file->symbol_table.getType(*name);
+  int symb_id = mod_file->symbol_table.getID(*name);
+  SymbolType type = mod_file->symbol_table.getType(symb_id);
 
   if (type != eEndogenous
       && type != eExogenous
@@ -330,7 +317,7 @@ ParsingDriver::hist_val(string *name, string *lag, NodeID rhs)
     error("hist_val: " + *name + " should be an endogenous or exogenous variable");
 
   int ilag = atoi(lag->c_str());
-  pair<string, int> key(*name, ilag);
+  pair<int, int> key(symb_id, ilag);
 
   if (hist_values.find(key) != hist_values.end())
     error("hist_val: (" + *name + ", " + *lag + ") declared twice");
@@ -345,14 +332,15 @@ void
 ParsingDriver::homotopy_val(string *name, NodeID val1, NodeID val2)
 {
   check_symbol_existence(*name);
-  Type type = mod_file->symbol_table.getType(*name);
+  int symb_id = mod_file->symbol_table.getID(*name);
+  SymbolType type = mod_file->symbol_table.getType(symb_id);
 
   if (type != eParameter
       && type != eExogenous
       && type != eExogenousDet)
     error("homotopy_val: " + *name + " should be a parameter or exogenous variable");
 
-  homotopy_values.push_back(make_pair(*name, make_pair(val1, val2)));
+  homotopy_values.push_back(make_pair(symb_id, make_pair(val1, val2)));
 
   delete name;
 }
@@ -368,21 +356,37 @@ ParsingDriver::forecast()
 void
 ParsingDriver::use_dll()
 {
-  model_tree->mode = eDLLMode;
+  mod_file->use_dll = true;
 }
 
 void
-ParsingDriver::sparse_dll()
+ParsingDriver::block()
 {
-  model_tree->mode = eSparseDLLMode;
-  model_tree->block_triangular.init_incidence_matrix(mod_file->symbol_table.endo_nbr);
+  mod_file->block = true;
 }
 
 void
-ParsingDriver::sparse()
+ParsingDriver::byte_code()
 {
-  model_tree->mode = eSparseMode;
-  model_tree->block_triangular.init_incidence_matrix(mod_file->symbol_table.endo_nbr);
+  mod_file->byte_code = true;
+}
+
+void
+ParsingDriver::cutoff(string *value)
+{
+  double val = atof(value->c_str());
+  mod_file->dynamic_model.cutoff = val;
+  mod_file->static_model.cutoff = val;
+  delete value;
+}
+
+void
+ParsingDriver::mfs(string *value)
+{
+  int val = atoi(value->c_str());
+  mod_file->dynamic_model.mfs = val;
+  mod_file->static_model.mfs = val;
+  delete value;
 }
 
 void
@@ -416,7 +420,7 @@ ParsingDriver::end_homotopy()
 void
 ParsingDriver::begin_model()
 {
-  set_current_data_tree(&mod_file->model_tree);
+  set_current_data_tree(&mod_file->dynamic_model);
 }
 
 void
@@ -434,32 +438,36 @@ ParsingDriver::end_shocks()
 void
 ParsingDriver::end_mshocks()
 {
-  mod_file->addStatement(new MShocksStatement(det_shocks, var_shocks, std_shocks,
-                                              covar_shocks, corr_shocks, mod_file->symbol_table));
+  mod_file->addStatement(new MShocksStatement(det_shocks, mod_file->symbol_table));
   det_shocks.clear();
-  var_shocks.clear();
-  std_shocks.clear();
-  covar_shocks.clear();
-  corr_shocks.clear();
 }
 
 void
-ParsingDriver::add_det_shock(string *var)
+ParsingDriver::add_det_shock(string *var, bool conditional_forecast)
 {
   check_symbol_existence(*var);
-  Type type = mod_file->symbol_table.getType(*var);
-  if (type != eExogenous && type != eExogenousDet)
-    error("shocks: shocks can only be applied to exogenous variables");
+  SymbolType type = mod_file->symbol_table.getType(*var);
+
+  if (conditional_forecast)
+    {
+      if (type != eEndogenous)
+        error("conditional_forecast_paths: shocks can only be applied to exogenous variables");
+    }
+  else
+    {
+      if (type != eExogenous && type != eExogenousDet)
+        error("shocks: shocks can only be applied to exogenous variables");
+    }
 
   if (det_shocks.find(*var) != det_shocks.end())
-    error("shocks: variable " + *var + " declared twice");
+    error("shocks/conditional_forecast_paths: variable " + *var + " declared twice");
 
   if (det_shocks_periods.size() != det_shocks_values.size())
-    error("shocks: variable " + *var + ": number of periods is different from number of shock values");
+    error("shocks/conditional_forecast_paths: variable " + *var + ": number of periods is different from number of shock values");
 
   vector<ShocksStatement::DetShockElement> v;
 
-  for(unsigned int i = 0; i < det_shocks_periods.size(); i++)
+  for (unsigned int i = 0; i < det_shocks_periods.size(); i++)
     {
       ShocksStatement::DetShockElement dse;
       dse.period1 = det_shocks_periods[i].first;
@@ -548,6 +556,8 @@ ParsingDriver::add_period(string *p1, string *p2)
 {
   int p1_val = atoi(p1->c_str());
   int p2_val = atoi(p2->c_str());
+  if (p1_val > p2_val)
+    error("shocks/conditional_forecast_paths: can't have first period index greater than second index in range specification");
   det_shocks_periods.push_back(make_pair(p1_val, p2_val));
   delete p1;
   delete p2;
@@ -573,14 +583,95 @@ ParsingDriver::add_value(string *p1)
   det_shocks_values.push_back(add_constant(p1));
 }
 
+void
+ParsingDriver::end_svar_identification()
+{
+  mod_file->addStatement(new SvarIdentificationStatement(svar_ident_exclusion_values,
+                                                         svar_upper_cholesky,
+                                                         svar_lower_cholesky,
+                                                         mod_file->symbol_table));
+  svar_upper_cholesky = false;
+  svar_lower_cholesky = false;
+  svar_restriction_symbols.clear();
+  svar_equation_restrictions.clear();
+  svar_ident_exclusion_values.clear();
+}
+
+void
+ParsingDriver::combine_lag_and_restriction(string *lag)
+{
+  int current_lag = atoi(lag->c_str());
+
+  for (SvarIdentificationStatement::svar_identification_exclusion_type::const_iterator it = svar_ident_exclusion_values.begin();
+       it != svar_ident_exclusion_values.end(); it++)
+    if (it->first.first == current_lag)
+      error("lag " + *lag + " used more than once.");
+
+  for (map<int, vector<int> >::const_iterator it = svar_equation_restrictions.begin();
+       it != svar_equation_restrictions.end(); it++)
+    svar_ident_exclusion_values[make_pair(current_lag, it->first)] = it->second;
+
+  svar_upper_cholesky = false;
+  svar_lower_cholesky = false;
+  svar_equation_restrictions.clear();
+  delete lag;
+}
+
+void
+ParsingDriver::add_restriction_in_equation(string *equation)
+{
+  int eqn = atoi(equation->c_str());
+  if (eqn < 1)
+    error("equation numbers must be greater than or equal to 1.");
+
+  if (svar_equation_restrictions.count(eqn) > 0)
+    error("equation number " + *equation + " referenced more than once under a single lag.");
+
+  svar_equation_restrictions[eqn] = svar_restriction_symbols;
+
+  svar_restriction_symbols.clear();
+  delete equation;
+}
+
+void
+ParsingDriver::add_in_svar_restriction_symbols(string *tmp_var)
+{
+  check_symbol_existence(*tmp_var);
+  int symb_id = mod_file->symbol_table.getID(*tmp_var);
+
+  for (vector<int>::const_iterator viit = svar_restriction_symbols.begin();
+       viit != svar_restriction_symbols.end(); viit++)
+    if (symb_id == *viit)
+      error(*tmp_var + " restriction added twice.");
+
+  svar_restriction_symbols.push_back(symb_id);
+  delete tmp_var;
+}
+
+void
+ParsingDriver::add_upper_cholesky()
+{
+  svar_upper_cholesky = true;
+  svar_lower_cholesky = false;
+}
+
+void
+ParsingDriver::add_lower_cholesky()
+{
+  svar_upper_cholesky = false;
+  svar_lower_cholesky = true;
+}
+
 void
 ParsingDriver::do_sigma_e()
 {
+  warning("Sigma_e: this command is now deprecated and may be removed in a future version of Dynare. Please use the \"shocks\" command instead.");
+
   try
     {
       mod_file->addStatement(new SigmaeStatement(sigmae_matrix));
     }
-  catch(SigmaeStatement::MatrixFormException &e)
+  catch (SigmaeStatement::MatrixFormException &e)
     {
       error("Sigma_e: matrix is neither upper triangular nor lower triangular");
     }
@@ -609,10 +700,7 @@ ParsingDriver::add_to_row(NodeID v)
 void
 ParsingDriver::steady()
 {
-  if (mod_file->model_tree.mode == eSparseMode)
-    mod_file->addStatement(new SteadySparseStatement(options_list));
-  else
-    mod_file->addStatement(new SteadyStatement(options_list));
+  mod_file->addStatement(new SteadyStatement(options_list));
   options_list.clear();
 }
 
@@ -638,16 +726,9 @@ ParsingDriver::option_num(const string &name_option, string *opt)
 void
 ParsingDriver::option_num(const string &name_option, const string &opt)
 {
-  // Since "periods" option automatically sets "simul" option, we don't want to fail if user explicitly sets both "simul" and "periods"
-  if (name_option != "simul"
-      && (options_list.num_options.find(name_option) != options_list.num_options.end()))
+  if (options_list.num_options.find(name_option) != options_list.num_options.end())
     error("option " + name_option + " declared twice");
 
-  if ((name_option == "periods") && (mod_file->model_tree.mode == eSparseDLLMode || mod_file->model_tree.mode == eSparseMode))
-    mod_file->model_tree.block_triangular.periods = atoi(opt.c_str());
-  else if (name_option == "cutoff")
-    mod_file->model_tree.cutoff = atof(opt.c_str());
-
   options_list.num_options[name_option] = opt;
 }
 
@@ -679,6 +760,20 @@ ParsingDriver::option_symbol_list(const string &name_option)
   symbol_list.clear();
 }
 
+void
+ParsingDriver::option_vec_int(const string &name_option, const vector<int> *opt)
+{
+  if (options_list.vector_int_options.find(name_option)
+      != options_list.vector_int_options.end())
+    error("option " + name_option + " declared twice");
+
+  if ((*opt).empty())
+    error("option " + name_option + " was passed an empty vector.");
+
+  options_list.vector_int_options[name_option] = *opt;
+  delete opt;
+}
+
 void
 ParsingDriver::linear()
 {
@@ -694,41 +789,22 @@ ParsingDriver::add_in_symbol_list(string *tmp_var)
   delete tmp_var;
 }
 
-void ParsingDriver::rplot()
+void
+ParsingDriver::rplot()
 {
   mod_file->addStatement(new RplotStatement(symbol_list, options_list));
   options_list.clear();
   symbol_list.clear();
 }
 
-void ParsingDriver::stoch_simul()
+void
+ParsingDriver::stoch_simul()
 {
   mod_file->addStatement(new StochSimulStatement(symbol_list, options_list));
   symbol_list.clear();
   options_list.clear();
 }
 
-void ParsingDriver::simulate()
-{
-  if (mod_file->model_tree.mode == eSparseDLLMode || mod_file->model_tree.mode == eSparseMode)
-    simul_sparse();
-  else
-    simul();
-}
-
-void
-ParsingDriver::simul_sparse()
-{
-  mod_file->addStatement(new SimulSparseStatement(options_list, mod_file->model_tree.compiler, mod_file->model_tree.mode));
-  options_list.clear();
-}
-
-void
-ParsingDriver::init_compiler(int compiler_type)
-{
-  mod_file->model_tree.compiler = compiler_type;
-}
-
 void
 ParsingDriver::simul()
 {
@@ -736,7 +812,6 @@ ParsingDriver::simul()
   options_list.clear();
 }
 
-
 void
 ParsingDriver::model_info()
 {
@@ -744,7 +819,6 @@ ParsingDriver::model_info()
   options_list.clear();
 }
 
-
 void
 ParsingDriver::check()
 {
@@ -760,7 +834,7 @@ ParsingDriver::add_estimated_params_element()
     check_symbol_existence(estim_params.name2);
 
   estim_params_list.push_back(estim_params);
-  estim_params.clear();
+  estim_params.init(*data_tree);
 }
 
 void
@@ -799,22 +873,6 @@ ParsingDriver::run_estimation()
   options_list.clear();
 }
 
-void
-ParsingDriver::run_prior_analysis()
-{
-  mod_file->addStatement(new PriorAnalysisStatement(symbol_list, options_list));
-  symbol_list.clear();
-  options_list.clear();
-}
-
-void
-ParsingDriver::run_posterior_analysis()
-{
-  mod_file->addStatement(new PosteriorAnalysisStatement(symbol_list, options_list));
-  symbol_list.clear();
-  options_list.clear();
-}
-
 void
 ParsingDriver::dynare_sensitivity()
 {
@@ -1011,29 +1069,50 @@ ParsingDriver::run_calib(int covar)
 }
 
 void
-ParsingDriver::run_dynatype(string *filename, string *ext)
+ParsingDriver::run_dynatype(string *filename)
 {
-  mod_file->addStatement(new DynaTypeStatement(symbol_list, *filename, *ext));
+  mod_file->addStatement(new DynaTypeStatement(symbol_list, *filename));
   symbol_list.clear();
   delete filename;
-  delete ext;
 }
 
 void
-ParsingDriver::run_dynasave(string *filename, string *ext)
+ParsingDriver::run_dynasave(string *filename)
 {
-  mod_file->addStatement(new DynaSaveStatement(symbol_list, *filename, *ext));
+  mod_file->addStatement(new DynaSaveStatement(symbol_list, *filename));
   symbol_list.clear();
   delete filename;
-  delete ext;
+}
+
+void
+ParsingDriver::run_load_params_and_steady_state(string *filename)
+{
+  mod_file->addStatement(new LoadParamsAndSteadyStateStatement(*filename, mod_file->symbol_table));
+  delete filename;
+}
+
+void
+ParsingDriver::run_save_params_and_steady_state(string *filename)
+{
+  mod_file->addStatement(new SaveParamsAndSteadyStateStatement(*filename));
+  delete filename;
+}
+
+void
+ParsingDriver::run_identification()
+{
+  mod_file->addStatement(new IdentificationStatement(options_list));
+  options_list.clear();
 }
 
 void
 ParsingDriver::add_mc_filename(string *filename, string *prior)
 {
-  if (filename_list.find(*filename) != filename_list.end())
-    error("model_comparison: filename " + *filename + " declared twice");
-  filename_list[*filename] = *prior;
+  for (ModelComparisonStatement::filename_list_type::iterator it = filename_list.begin();
+       it != filename_list.end(); it++)
+    if ((*it).first == *filename)
+      error("model_comparison: filename " + *filename + " declared twice");
+  filename_list.push_back(make_pair(*filename, *prior));
   delete filename;
   delete prior;
 }
@@ -1049,7 +1128,7 @@ ParsingDriver::run_model_comparison()
 void
 ParsingDriver::begin_planner_objective()
 {
-  set_current_data_tree(new ModelTree(mod_file->symbol_table, mod_file->num_constants));
+  set_current_data_tree(new StaticModel(mod_file->symbol_table, mod_file->num_constants));
 }
 
 void
@@ -1059,7 +1138,7 @@ ParsingDriver::end_planner_objective(NodeID expr)
   NodeID eq = model_tree->AddEqual(expr, model_tree->Zero);
   model_tree->addEquation(eq);
 
-  mod_file->addStatement(new PlannerObjectiveStatement(model_tree));
+  mod_file->addStatement(new PlannerObjectiveStatement(dynamic_cast<StaticModel *>(model_tree)));
 
   reset_data_tree();
 }
@@ -1072,6 +1151,18 @@ ParsingDriver::ramsey_policy()
   options_list.clear();
 }
 
+void
+ParsingDriver::write_latex_dynamic_model()
+{
+  mod_file->addStatement(new WriteLatexDynamicModelStatement(mod_file->dynamic_model));
+}
+
+void
+ParsingDriver::write_latex_static_model()
+{
+  mod_file->addStatement(new WriteLatexStaticModelStatement(mod_file->static_model));
+}
+
 void
 ParsingDriver::bvar_density(string *maxnlags)
 {
@@ -1088,6 +1179,136 @@ ParsingDriver::bvar_forecast(string *nlags)
   delete nlags;
 }
 
+void
+ParsingDriver::sbvar()
+{
+  mod_file->addStatement(new SBVARStatement(options_list));
+  options_list.clear();
+}
+
+void
+ParsingDriver::ms_sbvar()
+{
+  mod_file->addStatement(new MS_SBVARStatement(options_list));
+  options_list.clear();
+}
+
+void
+ParsingDriver::svar()
+{
+  OptionsList::num_options_type::const_iterator it0, it1, it2;
+  OptionsList::vec_int_options_type::const_iterator itv;
+
+  it0 = options_list.string_options.find("ms.coefficients");
+  it1 = options_list.string_options.find("ms.variances");
+  it2 = options_list.string_options.find("ms.constants");
+  if (it0 == options_list.string_options.end()
+      && it1 == options_list.string_options.end()
+      && it2 == options_list.string_options.end())
+    error("You must pass one of 'coefficients', 'variances', or 'constants'.");
+
+  if ((it0 != options_list.string_options.end()
+       && it1 != options_list.string_options.end())
+      || (it1 != options_list.string_options.end()
+          && it2 != options_list.string_options.end())
+      || (it0 != options_list.string_options.end()
+          && it2 != options_list.string_options.end()))
+    error("You may only pass one 'coefficients', 'variances', or 'constants' option.");
+
+  it0 = options_list.num_options.find("ms.chain");
+  if (it0 == options_list.num_options.end())
+    error("A chain option must be passed to the svar statement.");
+  else if (atoi(it0->second.c_str()) <= 0)
+    error("The value passed to the chain option must be greater than zero.");
+
+  itv = options_list.vector_int_options.find("ms.equations");
+  if (itv != options_list.vector_int_options.end())
+    for (vector<int>::const_iterator viit = itv->second.begin(); viit != itv->second.end(); viit++)
+      if (*viit <= 0)
+        error("The value(s) passed to the equation option must be greater than zero.");
+
+  mod_file->addStatement(new SvarStatement(options_list));
+  options_list.clear();
+}
+
+void
+ParsingDriver::markov_switching()
+{
+  OptionsList::num_options_type::const_iterator it0, it1;
+
+  it0 = options_list.num_options.find("ms.chain");
+  if (it0 == options_list.num_options.end())
+    error("A chain option must be passed to the markov_switching statement.");
+  else if (atoi(it0->second.c_str()) <= 0)
+    error("The value passed to the chain option must be greater than zero.");
+
+  it0 = options_list.num_options.find("ms.state");
+  it1 = options_list.num_options.find("ms.number_of_states");
+  if ((it0 == options_list.num_options.end())
+      && (it1 == options_list.num_options.end()))
+    error("Either a state option or a number_of_states option must be passed to the markov_switching statement.");
+
+  if ((it0 != options_list.num_options.end())
+      && (it1 != options_list.num_options.end()))
+    error("You cannot pass both a state option and a number_of_states option to the markov_switching statement.");
+
+  if (it0 != options_list.num_options.end())
+    if (atoi(it0->second.c_str()) <= 0)
+      error("The value passed to the state option must be greater than zero.");
+
+  if (it1 != options_list.num_options.end())
+    if (atoi(it1->second.c_str()) <= 0)
+      error("The value passed to the number_of_states option must be greater than zero.");
+
+  string infStr("Inf");
+  it0 = options_list.num_options.find("ms.duration");
+  if (it0 == options_list.num_options.end())
+    error("A duration option must be passed to the markov_switching statement.");
+  else if (infStr.compare(it0->second) != 0)
+    if (atof(it0->second.c_str()) <= 0.0)
+      error("The value passed to the duration option must be greater than zero.");
+
+  mod_file->addStatement(new MarkovSwitchingStatement(options_list));
+  options_list.clear();
+}
+
+void
+ParsingDriver::shock_decomposition()
+{
+  mod_file->addStatement(new ShockDecompositionStatement(symbol_list, options_list));
+  symbol_list.clear();
+  options_list.clear();
+}
+
+void
+ParsingDriver::conditional_forecast()
+{
+  mod_file->addStatement(new ConditionalForecastStatement(options_list));
+  options_list.clear();
+}
+
+void
+ParsingDriver::plot_conditional_forecast(string *periods)
+{
+  int nperiods;
+  if (periods == NULL)
+    nperiods = -1;
+  else
+    {
+      nperiods = atoi(periods->c_str());
+      delete periods;
+    }
+  mod_file->addStatement(new PlotConditionalForecastStatement(nperiods, symbol_list));
+  symbol_list.clear();
+}
+
+void
+ParsingDriver::conditional_forecast_paths()
+{
+  mod_file->addStatement(new ConditionalForecastPathsStatement(det_shocks, mod_file->symbol_table));
+  det_shocks.clear();
+}
+
 NodeID
 ParsingDriver::add_model_equal(NodeID arg1, NodeID arg2)
 {
@@ -1109,15 +1330,43 @@ ParsingDriver::declare_and_init_model_local_variable(string *name, NodeID rhs)
     {
       mod_file->symbol_table.addSymbol(*name, eModelLocalVariable);
     }
-  catch(SymbolTable::AlreadyDeclaredException &e)
+  catch (SymbolTable::AlreadyDeclaredException &e)
     {
       error("Local model variable " + *name + " declared twice.");
     }
 
-  model_tree->AddLocalParameter(*name, rhs);
+  model_tree->AddLocalVariable(*name, rhs);
   delete name;
 }
 
+void
+ParsingDriver::change_type(SymbolType new_type, vector<string *> *var_list)
+{
+  for (vector<string *>::iterator it = var_list->begin();
+       it != var_list->end(); it++)
+    {
+      int id;
+      try
+        {
+          id = mod_file->symbol_table.getID(**it);
+        }
+      catch (SymbolTable::UnknownSymbolNameException &e)
+        {
+          error("Unknown variable " + **it);
+        }
+
+      // Check if symbol already used in a VariableNode
+      if (mod_file->expressions_tree.isSymbolUsed(id)
+          || mod_file->dynamic_model.isSymbolUsed(id))
+        error("You cannot modify the type of symbol " + **it + " after having used it in an expression");
+
+      mod_file->symbol_table.changeType(id, new_type);
+
+      delete *it;
+    }
+  delete var_list;
+}
+
 NodeID
 ParsingDriver::add_plus(NodeID arg1, NodeID arg2)
 {
@@ -1190,6 +1439,14 @@ ParsingDriver::add_power(NodeID arg1, NodeID arg2)
   return data_tree->AddPower(arg1, arg2);
 }
 
+NodeID
+ParsingDriver::add_expectation(string *arg1, NodeID arg2)
+{
+  NodeID expectationNode = data_tree->AddExpectation(atoi(arg1->c_str()), arg2);
+  delete arg1;
+  return expectationNode;
+}
+
 NodeID
 ParsingDriver::add_exp(NodeID arg1)
 {
@@ -1229,85 +1486,104 @@ ParsingDriver::add_tan(NodeID arg1)
 NodeID
 ParsingDriver::add_acos(NodeID arg1)
 {
-  return data_tree->AddACos(arg1);
+  return data_tree->AddAcos(arg1);
 }
 
 NodeID
 ParsingDriver::add_asin(NodeID arg1)
 {
-  return data_tree->AddASin(arg1);
+  return data_tree->AddAsin(arg1);
 }
 
 NodeID
 ParsingDriver::add_atan(NodeID arg1)
 {
-  return data_tree->AddATan(arg1);
+  return data_tree->AddAtan(arg1);
 }
 
 NodeID
 ParsingDriver::add_cosh(NodeID arg1)
 {
-  return data_tree->AddCosH(arg1);
+  return data_tree->AddCosh(arg1);
 }
 
 NodeID
 ParsingDriver::add_sinh(NodeID arg1)
 {
-  return data_tree->AddSinH(arg1);
+  return data_tree->AddSinh(arg1);
 }
 
 NodeID
 ParsingDriver::add_tanh(NodeID arg1)
 {
-  return data_tree->AddTanH(arg1);
+  return data_tree->AddTanh(arg1);
 }
 
 NodeID
 ParsingDriver::add_acosh(NodeID arg1)
 {
-  return data_tree->AddACosH(arg1);
+  return data_tree->AddAcosh(arg1);
 }
 
 NodeID
 ParsingDriver::add_asinh(NodeID arg1)
 {
-  return data_tree->AddASinH(arg1);
+  return data_tree->AddAsinh(arg1);
 }
 
 NodeID
 ParsingDriver::add_atanh(NodeID arg1)
 {
-  return data_tree->AddATanH(arg1);
+  return data_tree->AddAtanh(arg1);
 }
 
 NodeID
 ParsingDriver::add_sqrt(NodeID arg1)
 {
-  return data_tree->AddSqRt(arg1);
+  return data_tree->AddSqrt(arg1);
 }
 
 NodeID
 ParsingDriver::add_max(NodeID arg1, NodeID arg2)
 {
-  return data_tree->AddMaX(arg1,arg2);
+  return data_tree->AddMax(arg1, arg2);
 }
 
 NodeID
 ParsingDriver::add_min(NodeID arg1, NodeID arg2)
 {
-  return data_tree->AddMin(arg1,arg2);
+  return data_tree->AddMin(arg1, arg2);
 }
 
 NodeID
 ParsingDriver::add_normcdf(NodeID arg1, NodeID arg2, NodeID arg3)
 {
-  return data_tree->AddNormcdf(arg1,arg2,arg3);
+  return data_tree->AddNormcdf(arg1, arg2, arg3);
+}
+
+NodeID
+ParsingDriver::add_normcdf(NodeID arg)
+{
+  return add_normcdf(arg, data_tree->Zero, data_tree->One);
+}
+
+NodeID
+ParsingDriver::add_steady_state(NodeID arg1)
+{
+  return data_tree->AddSteadyState(arg1);
+}
+
+void
+ParsingDriver::push_unknown_function_arg_vector_onto_stack()
+{
+  vector<NodeID> emptyvec;
+  stack_unknown_function_args.push(emptyvec);
 }
 
 void
 ParsingDriver::add_unknown_function_arg(NodeID arg)
 {
-  unknown_function_args.push_back(arg);
+  stack_unknown_function_args.top().push_back(arg);
 }
 
 NodeID
@@ -1321,8 +1597,8 @@ ParsingDriver::add_unknown_function(string *function_name)
   else
     mod_file->symbol_table.addSymbol(*function_name, eUnknownFunction);
 
-  NodeID id = data_tree->AddUnknownFunction(*function_name, unknown_function_args);
-  unknown_function_args.clear();
+  NodeID id = data_tree->AddUnknownFunction(*function_name, stack_unknown_function_args.top());
+  stack_unknown_function_args.pop();
   return id;
 }
 
